@@ -7,7 +7,7 @@ from scipy.sparse import csr_matrix, issparse, lil_matrix, csc_matrix
 
 from Orange.statistics.util import bincount, countnans, contingency, digitize, \
     mean, nanmax, nanmean, nanmedian, nanmin, nansum, nanunique, stats, std, \
-    unique, var
+    unique, var, nanstd, nanvar
 
 
 def dense_sparse(test_case):
@@ -192,6 +192,14 @@ class TestUtil(unittest.TestCase):
                     np.var(data, axis=axis)
                 )
 
+    @dense_sparse
+    def test_nanvar(self, array):
+        for X in self.data:
+            X_sparse = array(X)
+            np.testing.assert_array_equal(
+                nanvar(X_sparse),
+                np.nanvar(X))
+
     def test_std(self):
         for data in self.data:
             for axis in chain((None,), range(len(data.shape))):
@@ -201,6 +209,14 @@ class TestUtil(unittest.TestCase):
                     np.std(data, axis=axis)
                 )
 
+    @dense_sparse
+    def test_nanstd(self, array):
+        for X in self.data:
+            X_sparse = array(X)
+            np.testing.assert_array_equal(
+                nanstd(X_sparse),
+                np.nanstd(X))
+
 
 class TestDigitize(unittest.TestCase):
     def setUp(self):
